;*======================================================================*
;* TITLE	: VIDEO.INC						*
;* Function	: Video Definitions					*
;*									*
;* Project #	: JAGUAR						*
;* Programmer	: Andrew J. Burgess					*
;*									*
;* MODIFIED	: Pradip K Fatehpuria	10/27/93			*
;*									*
;* COPYRIGHT(c) 1992,1993 Atari Computer Corporation			*
;* UNATHORIZED REPRODUCTION, ADAPTATION, DISTRIBUTION, PERFORMANCE OR	*
;* DISPLAY OF THIS COMPUTER PROGRAM OR THE ASSOCIATED AUDIOVISUAL WORK	*
;* IS STRICTLY PROHIBITED.						*
;* ALL RIGHTS RESERVED.							*
;*									*
;*======================================================================*

;*======================================================================*
;* VIDEO CLOCK								*
;*	PAL	26.594 MHz						*
;*	NTSC	16.591 MHz						*
;*									*
;* In JAGUAR system - Processor Clock Speed = Video Clock Speed		*
;* The Pixel (Dot) Clock is produced from Video Clock. The ratio between*
;* video and pixel clock is determined by high order bits of the VMODE	*
;* register. The possible value of the ratio are (1..8).		*
;* For both PAL and NTSC, the safe video area is about 40 usec wide.	*
;* In case of overscan it is 50 usec.					*
;*======================================================================*

;*======================================================================*
;*	NOTE THAT THE HORIZONTAL TIMING NUMBERS ARE FOR NTSC
;*
;*	vclk		equ	400	for 25.00 MHz
;*			equ	399	for 25.05 MHz
;*			equ	376	for 26.60 MHz
;*======================================================================*

vclk		equ	376		; for 26.60 MHz
ivclk		equ	vclk

;*======================================================================*
;*	THE FOLLOWING ARE TIME IN UNITS OF 100 PS			*
;*======================================================================*

horiz_per	equ	317778 		; 63.5555us/2 Horizontal period
sync_per	equ	46050		; 4.605us synch width
eq_per		equ	23500		; 2.35us equalization
front_porch	equ	17450		; 1.745us 
line_blank	equ	109500		; 10.95us Total blanking

;*======================================================================*
;*	THIS IS THE DISPLAY WIDTH IN VIDEO CLOCKS
;*
;*	disp_width	equ	1347	<- (448 + 1) * 3 for SCRNORM
;*			equ	1796	<- (448 + 1) * 4 for SCROVER
;*			equ	963	<- (320 + 1) * 3 for SCRNORM
;*			equ	1284	<- (320 + 1) * 4 for SCROVER
;*			equ	1280	<- Leonard advocates
;*			equ	1356	<- Gives best center
;*			equ	1330
;*======================================================================*

disp_width	equ	1152	; this is in pixels/divider
disp_height	equ	200	; this is in lines

;*======================================================================*
;*	HORIZONTAL COMPUTATIONS						*
;*======================================================================*

n_hp		equ	horiz_per / vclk
n_hbb		equ	((horiz_per - sync_per - front_porch) / vclk) | $400
n_hbe		equ	(line_blank - sync_per - front_porch) / vclk
n_hs		equ	((horiz_per - sync_per) / vclk) | $400
n_hvs		equ	(horiz_per - (2 * sync_per)) / vclk
n_heq		equ	(horiz_per - eq_per) / vclk
n_hde		equ	((disp_width / 2) - 1) | $400
n_hdb1		equ	n_hp - (disp_width / 2)
n_hdb2		equ	n_hdb1

;*======================================================================*
;*	VERTICAL COMPUTATIONS						*
;*======================================================================*

n_vp		equ	523
n_vee		equ	6
n_vbe		equ	40
n_vdb		equ	n_vbe + 26
n_vde		equ	n_vdb + (disp_height * 2)
n_vbb		equ	n_vde + 40
n_vs		equ	n_vp - 10
n_veb		equ	n_vs - n_vee

;*======================================================================*
;*======================================================================*

SCRNTOP		equ	n_vdb				; ypos of first line at top of screen
SCRNBOTTOM	equ	n_vdb+199			; xpos of first line below screen

SCRNLEFT	equ	0				; xpos of left edge of screen
SCRNRIGHT	equ	((disp_width/4)-1)		; xpos of right edge of screen

SCRNXCTR	equ	((SCRNRIGHT - SCRNLEFT) / 2)	; ypos of first line at top of screen
SCRNYCTR	equ	((SCRNBOTTOM - SCRNTOP) / 2)	; xpos of first line below screen

;*======================================================================*
;* EOF									*
;*======================================================================*
