;*******************************************************************************
;*
;*				CD_HOST.S
;*
;* This MC680x0 program sends data to the Host Port of the DSP56K in the
;* Falcon030 computer.  A/B ping-pong buffers are used.  It is critical
;* that the amount od data sent be a modulo number of the DSP buffer size,
;* otherwise data will be left in the DSP at the end of a file transfer.
;*
;* For example, given a 4 CD Block-size DSP buffer of 9408 bytes, an 18
;* sector 9216 byte file on hard disc must be padded up to insure
;* transmission by the DSP.
;*
;*	Assembled by... asm filename
;*
;*			REVISION HISTORY
;*
;* Rev	By   Date	Description of edit
;* """	"""  """""""""	"""""""""""""""""""
;* 100	PKF  18 MAR 94	From Host.s from Dave
;*
;* 200	PKF  15 JUL 94	This file is used for both instances where we make
;*			a maximum of two SCSI read calls or one read call.
;*
;*			The only difference is in the routine SndBufA.  Remove
;*			comments to turn it into the one SCSI read instance.
;*
;*******************************************************************************

;******************************************************************************
;* CONDITIONAL ASSEMBLY
;******************************************************************************

SEEKBUFF	equ	1

ONEREADS	equ	1

;******************************************************************************
;* LOCAL EQUATES
;******************************************************************************

MFP_ISRB	equ	$fffffa01+$10

DSPICR		equ	$ffffa200	; Interrupt Control Register
DSPCVR		equ	$ffffa201	; Command Vector Register
DSPISR		equ	$ffffa202	; Interrupt Status Register
DSPIVR		equ	$ffffa203	; Interrupt Vector Register

DSPTXH		equ	$ffffa205	; Transmit Byte High

G_data		equ	$FF8942		; R 0000Xddd and 0000Xiii i is the pin state

;******************************************************************************
;*	CODE SECTION
;******************************************************************************

	CSECT	aaaa,code

;******************************************************************************
;* ROUTINES ACCESSED FROM OTHER MODULES
;******************************************************************************

	IF SEEKBUFF

	XDEF	_BuffSet

	ENDC

	XDEF	_HostIni
	XDEF	_BuffIni

	XDEF	_SndBufA
	XDEF	_SndBufB

	IF SEEKBUFF

	XDEF	_SndBufC

	ENDC

	XDEF	_Lswap
	XDEF	_Wswap

;****************************************
;* Initialise buffer parameters		*
;****************************************

	 IF SEEKBUFF

_BuffSet:
	move.l	#0,d0			; clear d0 register
	move.w	d0,_C_maxno		; 0 items in C buffer
	move.l	d0,_C_rdind		; initialise read index in C buffer
	move.l	d0,_C_ldind		; initialise load index in C buffer
	move.l	4(sp),_C_limit		; maximum no of bytes in C buffer
	rts

	ENDC

;****************************************
;* initialize the DSP host port		*
;****************************************

_HostIni:
	move.b	#$80,DSPCVR		; clear host command bit
	move.b	#$80,DSPICR		; initialize port bit set
.DSP_go:
	move.l	#8,d0			; 'go' command
	bsr	_SendCmd		; send the command to DSP
.Fini_0:
	rts

;****************************************
;* initialise parameters		*
;****************************************

_BuffIni:
	move.l	#0,d0			; clear d0 register
.InitA:
	move.l	#_A_adata,d1
	add.l	#$10000,d1
	and.l	#$FFFF0000,d1
	move.l	d1,_A_data

	move.w	d0,_A_maxno		; 0 items in A buffer
	move.l	d0,_A_rdind		; initialise read index in A buffer

	IF ONEREADS

	move.l	d0,_A_ldind		; initialise load index in A buffer

	ENDC

.InitB:
	move.w	d0,_B_maxno		; 0 items in B buffer
.Fini_1:
	rts

;****************************************
;* send data buffer A to DSP HOST PORT	*
;****************************************

_SndBufA:
	movem.l	d2-d3/a0-a2,-(sp)		; save registers
.SaveAsr:
	move.w	sr,d2			; set IPL to 7 for no interrupts
	ori	#$0700,sr		; $0500, video off, works too

.ParmA:	move.w	_A_maxno,d1		; set number of words to send
	ror.w	#1,d1			; devide by 2 for number of words
	sub.w	#1,d1			; maxno-1 for dbra

.PtrsA:	move.l	_A_data,a0		; set buffer A pointer
	move.l	_A_rdind,d3		; starting index in A buffer
	adda.l	d3,a0			; starting pointer in A buffer

	move.l	#DSPISR,a1		; a1 = HP intr status register address
	move.l	#(DSPTXH-1),a2		; a2 = HP data register address
.XferWtA:
	btst	#1,(a1)			; wait for DSP to set TXDE to 1
	beq.s	.XferWtA		; if bit not set then wait
.XferA:	move.w	(a0)+,d0
	rol.l	#8,d0
	move.l	d0,(a2)			; now burst 3 bytes out of 1 long

	IF ONEREADS

.NextA:
	move.l	a0,d0
	bclr.l	#15,d0
	move.l	d0,a0

	ENDC

.LoopA:	dbra	d1,.XferWtA		; loop d1+1 times to send burst to DSP

	IF ONEREADS

.SaveA:
	move.l	#0,d0
	move.w	_A_maxno,d0
	add.l	d0,d3
	and.l	#$00007FFF,d3
	move.l	d3,_A_rdind		; save read index

	ENDC

.RstrAsr:
	move.w	d2,sr			; restore the old interrupt priority
	move.b	#%11011111,MFP_ISRB	; clear timer interrupt latch

.FiniA:	movem.l	(sp)+,d2-d3/a0-a2	; restore registers
	rts

;****************************************
;* send data buffer B to DSP HOST PORT	*
;****************************************

_SndBufB:
	movem.l	d2-d3/a0-a2,-(sp)	; save registers
.SaveBsr:
	move.w	sr,d2			; set IPL to 7 for no interrupts
	ori	#$0700,sr		; $0500, video off, works too

.ParmB:	move.w	_B_maxno,d1		; set  number of words to send
	ror.w	#1,d1
	sub.w	#1,d1			; maxno - 1 for dbra

	moveq.l	#0,d0			; clear d0

.PtrsB:	move.l	#DSPISR,a1		; a1 = HP interrupt status register
	move.l	#(DSPTXH-1),a2		; a2 = HP data register address
.XferWtB:
	btst	#1,(a1)			; wait for DSP to set TXDE to 1
	beq.s	.XferWtB		; if bit not set then wait
.XferB:	move.l	d0,(a2)			; now burst 3 bytes out of 1 long
	dbra	d1,.XferWtB		; loop d1+1 times to send burst to DSP

.RstrBsr:
	move.w	d2,sr			; restore the old interrupt priority
	move.b	#%11011111,MFP_ISRB	; clear timer interrupt latch

.FiniB:	movem.l	(sp)+,d2-d3/a0-a2	; restore registers
	rts

;****************************************
;* send data buffer C to DSP HOST PORT	*
;****************************************

	IF SEEKBUFF

_SndBufC:
	movem.l	d2-d3/a0-a2,-(sp)	; save registers
.SaveCsr:
	move.w	sr,d2			; set IPL to 7 for no interrupts
	ori	#$0700,sr		; $0500, video off, works too

.ParmC:	move.w	_C_maxno,d1		; set number of words to send
	ror.w	#1,d1			; devide by 2 for number of words
	sub.w	#1,d1			; maxno-1 for dbra

.PtrsC:	move.l	#_C_data,a0		; set buffer A pointer
	move.l	_C_rdind,d3		; starting index in A buffer
	adda.l	d3,a0			; starting pointer in A buffer

	move.l	#DSPISR,a1		; a1 = HP interrupt status register
	move.l	#(DSPTXH-1),a2		; a2 = HP data register address
.XferWtC:
	btst	#1,(a1)			; wait for DSP to set TXDE to 1
	beq.s	.XferWtC		; if bit not set then wait
.XferC:	move.w	(a0)+,d0
	rol.l	#8,d0
	move.l	d0,(a2)			; now burst 3 bytes out of 1 long

.LoopC:	dbra	d1,.XferWtC		; loop d1+1 times to send burst to DSP

.SaveC:	move.l	#0,d0
	move.w	_C_maxno,d0
	add.l	d0,d3
	move.l	d3,_C_rdind		; save read index

.RstrCsr:
	move.w	d2,sr			; restore the old interrupt priority
	move.b	#%11011111,MFP_ISRB	; clear timer interrupt latch

.FiniC:	movem.l	(sp)+,d2-d3/a0-a2	; restore registers
	rts

	ENDC

;****************************************
;* Send a command to DSP		*
;****************************************

_SendCmd:
	move.l	a0,-(sp)		; save registers
.s_wait:
	move.w	DSPISR,d1		; get contents of DSPISR register to d1
	btst	#9,d1			; check if bit 1 is set
	beq	.s_wait			; if not, wait
.nowdoit:
	move.l	#DSPICR,a0
	move.b	d0,(a0)			; send command from d0
.getack:				; get DSP's acknowledgement
	move.w	DSPISR,d0
.finish7:
	move.l	(sp)+,a0		; restore registers
	rts

;*******************************************************************************
;* _Lswap: read a long from MSDOS type disk image
;*******************************************************************************

_Lswap:
	movem.l	d1/a0,-(sp)

	move.l	12(sp),a0		; get the array pointer
	move.l	16(sp),d1		; get the offset into array
	add.l	d1,a0			; get the address of first byte

	moveq	#0,d0			; clear d0

	move.b	(a0)+,d0		; get first byte
	move.b	(a0)+,d1		; get second byte
	lsl.w	#8,d1			; left shift by 8 bits
	or.w	d1,d0			; get the low word in long
	swap	d0			; save it in high word

	move.b	(a0)+,d0		; get the third byte
	move.b	(a0)+,d1		; get the fourth byte
	lsl.w	#8,d1			; left whift by 8 bits
	or.w	d1,d0			; get the high word of long
	swap	d0			; put the words in proper place

	movem.l	(sp)+,d1/a0
	rts

;*******************************************************************************
;* _Wswap: read a long from MSDOS type disk image
;*******************************************************************************

_Wswap:
	movem.l	d1/a0,-(sp)

	move.l	12(sp),a0		; get the array pointer
	move.l	16(sp),d1		; get the offset into array
	add.l	d1,a0			; get the address of first byte

	moveq	#0,d0			; clear d0

	move.b	(a0)+,d0		; get first byte
	move.b	(a0)+,d1		; get second byte
	lsl.w	#8,d1			; left shift by 8 bits
	or.w	d1,d0			; get the low word in long

	movem.l	(sp)+,d1/a0
	rts

;******************************************************************************
;*	BSS SECTION
;******************************************************************************

	CSECT	bbbb,bss

;******************************************************************************
;* DATA ACCESSED FROM OTHER MODULES
;******************************************************************************

	XDEF	_A_maxno
	XDEF	_A_rdind

	IF ONEREADS

	XDEF	_A_ldind

	ENDC

	XDEF	_A_data

	XDEF	_B_maxno

	IF SEEKBUFF

	XDEF	_C_maxno
	XDEF	_C_rdind
	XDEF	_C_ldind
	XDEF	_C_limit
	XDEF	_C_data

	ENDC

;******************************************************************************
;* MEMORY BUFFERS USED HERE
;******************************************************************************

_A_maxno:	ds.w	1		; number of valid bytes in A buffer
_A_rdind:	ds.l	1		; read position in A buffer

	IF ONEREADS

_A_ldind:	ds.l	1		; load position in A buffer

	ENDC

_A_data:	ds.b	1		; pointer to file data buffer
_A_adata:	ds.b	$20000		; file data (32K) = twice the size of DSP buffer

_B_maxno:	ds.w	1		; number of valid bytes in B buffer

	IF SEEKBUFF

_C_maxno:	ds.w	1		; number of valid bytes in C buffer
_C_rdind:	ds.l	1		; read position in C buffer
_C_ldind:	ds.l	1		; load position in C buffer
_C_limit:	ds.l	1		; maximum number of bytes possible in C buffer
_C_data:	ds.b	$a00000		; buffer to be filled while seeking
_C_padd:	ds.b	$4000		; provides padding while reading 16K at a time

	ENDC

;******************************************************************************
;* END OF PRAGRAM
;******************************************************************************

	END

;*******************************************************************************
;* End of SNDBIND.S
;*******************************************************************************
